@@ -2,7 +2,7 @@ use 5.008007;
 package DBIx::Custom;
 use Object::Simple -base;
 
-our $VERSION = '0.39';
+our $VERSION = '0.40';
 
 use Carp 'croak';
 use DBI;
@@ -218,7 +218,7 @@ sub create_model {
 sub execute {
   my $self = shift;
   my $sql = shift;
-
+  
   # Options
   my $param;
   $param = shift if @_ % 2;
@@ -250,72 +250,79 @@ sub execute {
   $parsed_sql = $after_build_sql->($parsed_sql) if $after_build_sql;
   
   # Type rule
-  my $type_filters = {};
-  my $type_rule_off = !$self->{_type_rule_is_called} || $opt{type_rule_off};
-  unless ($type_rule_off) {
-    my $tables = $opt{table} || [];
-    $tables = [$tables] unless ref $tables eq 'ARRAY';
+  my $type_filters;
+  if ($self->{_type_rule_is_called}) {
+    $type_filters = {};
+    unless ($opt{type_rule_off}) {
+      my $tables = $opt{table} || [];
+      $tables = [$tables] unless ref $tables eq 'ARRAY';
 
-    # Tables
-    my $main_table = @{$tables}[-1];
-    
-    my $type_rule_off_parts = {
-      1 => $opt{type_rule1_off},
-      2 => $opt{type_rule2_off}
-    };
-    for my $i (1, 2) {
-      unless ($type_rule_off_parts->{$i}) {
-        $type_filters->{$i} = {};
-        my $table_alias = $opt{table_alias} || {};
-        for my $alias (keys %$table_alias) {
-          my $table = $table_alias->{$alias};
-          
-          for my $column (keys %{$self->{"_into$i"}{key}{$table} || {}}) {
-            $type_filters->{$i}->{"$alias.$column"} = $self->{"_into$i"}{key}{$table}{$column};
+      # Tables
+      my $main_table = @{$tables}[-1];
+      
+      my $type_rule_off_parts = {
+        1 => $opt{type_rule1_off},
+        2 => $opt{type_rule2_off}
+      };
+      for my $i (1, 2) {
+        unless ($type_rule_off_parts->{$i}) {
+          $type_filters->{$i} = {};
+          my $table_alias = $opt{table_alias} || {};
+          for my $alias (keys %$table_alias) {
+            my $table = $table_alias->{$alias};
+            
+            for my $column (keys %{$self->{"_into$i"}{key}{$table} || {}}) {
+              $type_filters->{$i}->{"$alias.$column"} = $self->{"_into$i"}{key}{$table}{$column};
+            }
           }
+          $type_filters->{$i} = {%{$type_filters->{$i}}, %{$self->{"_into$i"}{key}{$main_table} || {}}}
+            if $main_table;
         }
-        $type_filters->{$i} = {%{$type_filters->{$i}}, %{$self->{"_into$i"}{key}{$main_table} || {}}}
-          if $main_table;
       }
     }
   }
   
   # Replace filter name to code
-  my $filter = ref $opt{filter} eq 'ARRAY' ?
-    _array_to_hash($opt{filter}) : $opt{filter};
-  for my $column (keys %$filter) {
-    my $name = $filter->{$column};
-    if (!defined $name) {
-      $filter->{$column} = undef;
+  my $filter = $opt{filter};
+  if (defined $filter) {
+    if (ref $opt{filter} eq 'ARRAY') {
+      $filter = _array_to_hash($filter);
     }
-    elsif (ref $name ne 'CODE') {
-      croak qq{Filter "$name" is not registered" } . _subname
-        unless exists $self->filters->{$name};
-      $filter->{$column} = $self->filters->{$name};
-    }
-  }
-
-  # Check unsafe param names
-  unless ((join('', keys %$param) || '') =~ /^[$safe_char\.]+$/) {
-    for my $column (keys %$param) {
-      croak qq{"$column" is not safety column name } . _subname
-        unless $column =~ /^[$safe_char\.]+$/;
+    
+    for my $column (keys %$filter) {
+      my $name = $filter->{$column};
+      if (!defined $name) {
+        $filter->{$column} = undef;
+      }
+      elsif (ref $name ne 'CODE') {
+        croak qq{Filter "$name" is not registered" } . _subname
+          unless exists $self->filters->{$name};
+        $filter->{$column} = $self->filters->{$name};
+      }
     }
   }
   
+  # Bind type
+  my $bind_type = $opt{bind_type};
+  $bind_type = _array_to_hash($bind_type) if ref $bind_type eq 'ARRAY';
+  
+  # Fix param
+  
   # Create query
   my $query = DBIx::Custom::Query->new;
   $query->param($param);
   $query->sql($parsed_sql);
   $query->columns($columns);
-  $query->bind_type($opt{bind_type});
+  $query->bind_type($bind_type);
   
   $query->{_filter} = $filter;
   $query->{_type_filters} = $type_filters;
-  $query->{_type_rule_is_called} = $self->{_type_rule_is_called};
   $query->{_into1} = $self->{_into1};
   $query->{_into2} = $self->{_into2};
   
+  # Has filter
+  $query->{_f} = defined $filter || defined $type_filters;
+  
   # Build bind values
   $query->build;
 
@@ -337,7 +344,7 @@ sub execute {
   my $affected;
   eval {
     my $bind_values = $query->bind_values;
-    if ($opt{bind_type}) {
+    if ($bind_type) {
       my $bind_value_types = $query->bind_value_types;
       $sth->bind_param($_ + 1, $bind_values->[$_],
           $bind_value_types->[$_] ? $bind_value_types->[$_] : ())
@@ -488,25 +495,21 @@ sub mapper {
 }
 
 sub merge_param {
-  my ($self, @params) = @_;
+  my ($self, $param1, $param2) = @_;
   
   # Merge parameters
-  my $merge = {};
-  for my $param (@params) {
-    for my $column (keys %$param) {
-      my $param_is_array = ref $param->{$column} eq 'ARRAY' ? 1 : 0;
-      
-      if (exists $merge->{$column}) {
-        $merge->{$column} = [$merge->{$column}]
-          unless ref $merge->{$column} eq 'ARRAY';
-        push @{$merge->{$column}},
-          ref $param->{$column} ? @{$param->{$column}} : $param->{$column};
-      }
-      else { $merge->{$column} = $param->{$column} }
+  my $merged_param = {%$param1};
+  for my $column (keys %$param2) {
+    if (exists $merged_param->{$column}) {
+      $merged_param->{$column} = [$merged_param->{$column}]
+        unless ref $merged_param->{$column} eq 'ARRAY';
+      push @{$merged_param->{$column}},
+        ref $param2->{$column} eq 'ARRAY' ? @{$param2->{$column}} : $param2->{$column};
     }
+    else { $merged_param->{$column} = $param2->{$column} }
   }
   
-  return $merge;
+  return $merged_param;
 }
 
 sub model {
@@ -854,22 +857,23 @@ sub values_clause {
   # Create insert parameter tag
   my ($q, $p) = $self->_qp;
   
-  # values clause(performance is important)
-  '(' .
-  join(
-    ', ',
-    map { "$q$_$p" } sort keys %$param
-  ) .
-  ') values (' .
-  join(
-    ', ',
-    map {
-      ref $param->{$_} eq 'SCALAR' ? ${$param->{$_}} :
-      $wrap->{$_} ? $wrap->{$_}->(":$_") :
-      ":$_";
-    } sort keys %$param
-  ) .
-  ')'
+  my $safety_character = $self->safety_character;
+  
+  my @columns;
+  my @place_holders;
+  for my $column (keys %$param) {
+    croak qq{"$column" is not safety column name in values clause} . _subname
+      unless $column =~ /^[$safety_character\.]+$/;
+
+    push @columns, "$q$column$p";
+    push @place_holders, ref $param->{$column} eq 'SCALAR' ? ${$param->{$column}} :
+      $wrap->{$column} ? $wrap->{$column}->(":$column") :
+      ":$column";
+  }
+  
+  my $values_clause = '(' . join(', ', @columns) . ') values (' . join(', ', @place_holders) . ')';
+  
+  return $values_clause;
 }
 
 sub assign_clause {
@@ -877,16 +881,22 @@ sub assign_clause {
   
   my $wrap = $opts->{wrap} || {};
   my ($q, $p) = $self->_qp;
+
+  my $safety_character = $self->safety_character;
+
+  my @set_values;
+  for my $column (keys %$param) {
+    croak qq{"$column" is not safety column name in assign clause} . _subname
+      unless $column =~ /^[$safety_character\.]+$/;
+      
+    push @set_values, ref $param->{$column} eq 'SCALAR' ? "$q$column$p = " . ${$param->{$column}}
+      : $wrap->{$column} ? "$q$column$p = " . $wrap->{$column}->(":$column")
+      : "$q$column$p = :$column";
+  }
   
-  # Assign clause (performance is important)
-  join(
-    ', ',
-    map {
-      ref $param->{$_} eq 'SCALAR' ? "$q$_$p = " . ${$param->{$_}}
-      : $wrap->{$_} ? "$q$_$p = " . $wrap->{$_}->(":$_")
-      : "$q$_$p = :$_";
-    } sort keys %$param
-  );
+  my $assign_clause = join(', ', @set_values);
+  
+  return $assign_clause;
 }
 
 sub where { DBIx::Custom::Where->new(dbi => shift, @_) }
@@ -1179,20 +1189,35 @@ sub _multi_values_clause {
   # Create insert parameter tag
   my ($q, $p) = $self->_qp;
   
-  # Multi values clause
-  my $clause = '(' . join(', ', map { "$q$_$p" } sort keys %{$params->[0]}) . ') values ';
+  my $safety_character = $self->safety_character;
   
-  for (1 .. @$params) {
-    $clause .= '(' . join(', ', 
-      map {
-        ref $params->[0]->{$_} eq 'SCALAR' ? ${$params->[0]->{$_}} :
-        $wrap->{$_} ? $wrap->{$_}->(":$_") :
-        ":$_";
-      } sort keys %{$params->[0]}
-    ) . '), '
+  my $first_param = $params->[0];
+  
+  my @columns;
+  my @columns_quoted;
+  for my $column (keys %$first_param) {
+    croak qq{"$column" is not safety column name in multi values clause} . _subname
+      unless $column =~ /^[$safety_character\.]+$/;
+    
+    push @columns, $column;
+    push @columns_quoted, "$q$column$p";
   }
-  $clause =~ s/, $//;
-  return $clause;
+
+  # Multi values clause
+  my $multi_values_clause = '(' . join(', ', @columns_quoted) . ') values ';
+
+  for my $param (@$params) {
+    my @place_holders;
+    for my $column (@columns) {
+      push @place_holders, ref $param->{$column} eq 'SCALAR' ? ${$param->{$column}} :
+        $wrap->{$column} ? $wrap->{$column}->(":$column") :
+        ":$column";
+    }
+    $multi_values_clause .= '(' . join(', ', @place_holders) . '), ';
+  }
+  $multi_values_clause =~ s/, $//;
+  
+  return $multi_values_clause;
 }
 
 sub _id_to_param {
@@ -1382,11 +1407,17 @@ sub _where_clause_and_param {
   
   $where ||= {};
   my $w = {};
-
+  
   if (ref $where eq 'HASH') {
+    my $safety_character = $self->safety_character;
+    
     my $clause = [];
     my $column_join = '';
     for my $column (keys %$where) {
+      
+      croak qq{"$column" is not safety column name in where clause} . _subname
+        unless $column =~ /^[$safety_character\.]+$/;
+      
       $column_join .= $column;
       my $table;
       my $c;
