@@ -4,37 +4,11 @@ use Object::Simple -base;
 use Carp 'croak';
 use DBIx::Custom::Util qw/_subname _deprecate/;
 
-# Carp trust relationship
-push @DBIx::Custom::CARP_NOT, __PACKAGE__;
-
-has [qw/dbi table ctime mtime bind_type join primary_key/];
+has [qw/dbi table name ctime mtime bind_type join/];
 has columns => sub { [] };
 
 our $AUTOLOAD;
 
-sub AUTOLOAD {
-  my $self = shift;
-
-  # Method name
-  my ($package, $mname) = $AUTOLOAD =~ /^([\w\:]+)\:\:(\w+)$/;
-
-  # Method
-  $self->{_methods} ||= {};
-  if (my $method = $self->{_methods}->{$mname}) {
-    return $self->$method(@_)
-  }
-  elsif (my $dbi_method = $self->dbi->can($mname)) {
-    $self->dbi->$dbi_method(@_);
-  }
-  elsif ($self->{dbh} && (my $dbh_method = $self->dbh->can($mname))) {
-    $self->dbi->dbh->$dbh_method(@_);
-  }
-  else {
-    croak qq{Can't locate object method "$mname" via "$package" }
-      . _subname;
-  }
-}
-
 my @methods = qw(insert update update_all delete delete_all select count);
 for my $method (@methods) {
   
@@ -67,9 +41,21 @@ for my $method (@methods) {
   croak $code if $@;
 }
 
+# DEPRECATED
+sub primary_key {
+  if (@_ == 1) {
+    return $_[0]{'primary_key'};
+  }
+  $_[0]{'primary_key'} = $_[1];
+  $_[0];
+};
+
+# DEPRECATED
 sub update_or_insert {
   my ($self, $param, %opt) = @_;
-  
+
+  _deprecate('0.39', "DBIx::Custom::Model::update_or_insert method is DEPRECATED!");
+
   croak "update_or_insert method need primary_key and id option "
     unless (defined $opt{id} || defined $self->{id})
         && (defined $opt{primary_key} || defined $self->{primary_key});
@@ -85,11 +71,39 @@ sub update_or_insert {
   else { croak "selected row must be one " . _subname }
 }
 
+# DEPRECATED
+sub AUTOLOAD {
+  my $self = shift;
+  
+  _deprecate('0.39', "DBIx::Custom::Model AUTOLOAD feature is DEPRECATED!");
+  
+  # Method name
+  my ($package, $mname) = $AUTOLOAD =~ /^([\w\:]+)\:\:(\w+)$/;
+  
+  # Method
+  $self->{_methods} ||= {};
+  if (my $method = $self->{_methods}->{$mname}) {
+    return $self->$method(@_)
+  }
+  elsif (my $dbi_method = $self->dbi->can($mname)) {
+    $self->dbi->$dbi_method(@_);
+  }
+  elsif ($self->{dbh} && (my $dbh_method = $self->dbh->can($mname))) {
+    $self->dbi->dbh->$dbh_method(@_);
+  }
+  else {
+    croak qq{Can't locate object method "$mname" via "$package" }
+      . _subname;
+  }
+}
 sub DESTROY { }
 
+# DEPRECATED
 sub helper {
   my $self = shift;
   
+  _deprecate('0.39', "DBIx::Custom::Model::helper method is DEPRECATED!");
+  
   # Merge
   my $methods = ref $_[0] eq 'HASH' ? $_[0] : {@_};
   $self->{_methods} = {%{$self->{_methods} || {}}, %$methods};
@@ -143,19 +157,19 @@ my $model = DBIx::Custom::Model->new(table => 'books');
 
 =head1 ATTRIBUTES
 
-=head2 dbi
+=head2 name
 
-  my $dbi = $model->dbi;
-  $model = $model->dbi($dbi);
+  my $name = $model->name;
+  $model = $model->name('book');
 
-L<DBIx::Custom> object.
+Model name.
 
-=head2 ctime
+=head2 table
 
-  my $ctime = $model->ctime;
-  $model = $model->ctime('created_time');
+  my $table = $model->table;
+  $model = $model->table('book');
 
-Create timestamp column, this is passed to C<insert> or C<update> method.
+Table name, this is passed to C<insert>, C<update>, C<update_all>, C<delete>, C<delete_all>, C<select> method.
 
 =head2 join
 
@@ -166,20 +180,12 @@ Create timestamp column, this is passed to C<insert> or C<update> method.
   
 Join clause, this value is passed to C<select> method.
 
-=head2 primary_key
+=head2 dbi
 
-  my $primary_key = $model->primary_key;
-  $model = $model->primary_key(['id', 'number']);
+  my $dbi = $model->dbi;
+  $model = $model->dbi($dbi);
 
-Primary key,this is passed to C<insert>, C<update>,
-C<delete>, and C<select> method.
-
-=head2 table
-
-  my $model = $model->table;
-  $model = $model->table('book');
-
-Table name, this is passed to C<select> method.
+L<DBIx::Custom> object.
 
 =head2 bind_type
 
@@ -197,20 +203,27 @@ and C<select> method
 
 Updated timestamp column, this is passed to C<update> method.
 
+=head2 ctime
+
+  my $ctime = $model->ctime;
+  $model = $model->ctime('created_time');
+
+Create timestamp column, this is passed to C<insert> or C<update> method.
+
+=head2 primary_key
+
+  my $primary_key = $model->primary_key;
+  $model = $model->primary_key(['id', 'number']);
+
+Primary key,this is passed to C<insert>, C<update>,
+C<delete>, and C<select> method.
+
 =head1 METHODS
 
 L<DBIx::Custom::Model> inherits all methods from L<Object::Simple>,
 and you can use all methods of L<DBIx::Custom> and L<DBI>
 and implements the following new ones.
 
-=head2 count
-
-  my $count = $model->count;
-
-Get rows count.
-
-Options is same as C<select> method's ones.
-
 =head2 delete
 
   $model->delete(...);
@@ -232,26 +245,6 @@ you don't have to specify options if you set attribute in model.
 Same as C<insert> of L<DBIx::Custom> except that
 you don't have to specify options if you set attribute in model.
 
-=head2 helper
-
-  $model->helper(
-    update_or_insert => sub {
-      my $self = shift;
-      
-      # ...
-    },
-    find_or_create   => sub {
-      my $self = shift;
-      
-      # ...
-    }
-  );
-
-Register helper. These helper is called directly from L<DBIx::Custom::Model> object.
-
-  $model->update_or_insert;
-  $model->find_or_create;
-
 =head2 mycolumn
 
   my $column = $self->mycolumn;
@@ -293,11 +286,4 @@ you don't have to specify options if you set attribute in model.
 Same as C<update_all> of L<DBIx::Custom> except that
 you don't have to specify options if you set attribute in model.
 
-=head2 update_or_insert
-
-  $model->update_or_insert(...);
-  
-Same as C<update> of L<DBIx::Custom> except that
-you don't have to specify options if you set attribute in model.
-
 =cut
